/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef IDLC_CPP_GEN_CPP_GEN_BASE_H_
#define IDLC_CPP_GEN_CPP_GEN_BASE_H_

#include <memory>
#include <string>
#include <map>

#include "idlc/type.h"
#include "idlc/structure.h"
#include "idlc/generator.h"

namespace tidl {

class CppGeneratorBase : public Generator {
 public:
  explicit CppGeneratorBase(std::shared_ptr<Document> doc);
  virtual ~CppGeneratorBase() = default;

  void GenVersion(std::ofstream& stream);
  void GenStructuresForHeader(std::ofstream& stream);
  void GenStructuresForBody(std::ofstream& stream);
  void GenSerializer(std::ofstream& stream);
  void GenDeSerializer(std::ofstream& stream);
  void GenListSerializer(std::ofstream& stream, bool proto = false);
  void GenPrototype(std::ofstream& stream);
  void GenMethodId(std::ofstream& stream, const Interface& iface);
  void GenDelegateId(std::ofstream& stream, const Interface& iface);
  void GenParameters(std::ofstream& stream, const Parameters& ps);
  void GenDeclaration(std::ofstream& stream, const Interface& iface,
                      const Declaration& decl);
  void GenBodyCallbacks(std::ofstream& stream, const Interface& iface,
                        bool is_proxy);
  void GenHeaderCallbacks(std::ofstream& stream, const Interface& iface,
                          bool is_proxy);
  std::string ConvertTypeToString(const BaseType& type);
  std::string Tab(int cnt);
  std::string NLine(int cnt);
  std::string ConvertTypeToDeserializer(const BaseType& type,
                                        std::string id, std::string parcel,
                                        bool make_new_type = true);
  std::string ConvertTypeToSerializer(const BaseType& type,
                                      std::string id, std::string parcel);
  std::string GetParameters(const Parameters& ps);
  void GenLogTag(std::ofstream& stream, std::string id);
  void GenLogDefinition(std::ofstream& stream);

 private:
  void GenSetter(std::ofstream& stream, const Element& ele);
  void GenGetter(std::ofstream& stream, const Element& ele);
  void AddSerializerList(const BaseType& type);
  void GenListSerializer(std::ofstream& stream, const BaseType& type,
                         bool proto = false);
  void GenDeSerializer(std::ofstream& stream, const Structure& st,
                       bool proto = false);
  void GenSerializer(std::ofstream& stream, const Structure& st,
                     bool proto = false);
  void GenStructureForHeader(std::ofstream& stream, const Structure& st);
  void GenStructureForBody(std::ofstream& stream, const Structure& st);
  void GenBodyCallback(std::ofstream& stream, const Interface& iface,
                       const Declaration& decl, bool is_proxy);
  void GenHeaderCallback(std::ofstream& stream, const Declaration& decl,
                         bool is_proxy);

 protected:
  const int TAB_SIZE = 2;

 private:
  std::map<std::string, std::string> type_map_;
  std::map<std::string, std::string> parcel_type_map_;
  std::map<std::string, std::string> type_init_map_;
  std::map<std::string, const BaseType*> serializer_list_;
};

}  // namespace tidl

#endif  // IDLC_CPP_GEN_CPP_GEN_BASE_H_
