/*
 * Copyright (c) 2018 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef IDLC_CPP_GEN_CPP_STUB_BODY_GEN_CB_H_
#define IDLC_CPP_GEN_CPP_STUB_BODY_GEN_CB_H_

const char CB_CTOR_FRONT[] =
R"__cpp_cb(
##::##() {
  int r = rpc_port_stub_create(&stub_, "##");
  if (r != RPC_PORT_ERROR_NONE) {
    _E("Failed to create stub handle");
    throw InvalidIOException();
  }
  rpc_port_stub_add_connected_event_cb(stub_, OnConnectedCB, this);
  rpc_port_stub_add_disconnected_event_cb(stub_, OnDisconnectedCB, this);
  rpc_port_stub_add_received_event_cb(stub_, OnReceivedCB, this);
)__cpp_cb";

const char CB_DEFAULT_METHODS[] =
R"__cpp_cb(
##::~##() {
  for (auto& i : services_) {
    i->OnTerminate();
  }

  if (stub_) {
    rpc_port_stub_destroy(stub_);
  }
}

void ##::Listen(std::shared_ptr<##::ServiceBase::Factory> service_factory) {
  service_factory_ = std::move(service_factory);
  int r = rpc_port_stub_listen(stub_);
  if (r != RPC_PORT_ERROR_NONE) {
    _E("Failed to listen stub");
    switch (r) {
      case RPC_PORT_ERROR_INVALID_PARAMETER:
      case RPC_PORT_ERROR_IO_ERROR:
        throw InvalidIOException();
    }
  }
}

void ##::OnConnectedCB(const char* sender, const char* instance, void *data) {
  ##* stub = static_cast<##*>(data);
  auto s = stub->service_factory_->CreateService(sender, instance);
  s->OnCreate();
  stub->services_.emplace_back(std::move(s));
}

void ##::OnDisconnectedCB(const char* sender, const char* instance, void *data) {
  ##* stub = static_cast<##*>(data);

  for (auto& i : stub->services_) {
    if (i->GetInstance() == instance) {
      i->OnTerminate();
      stub->services_.remove(i);
      return;
    }
  }
}
)__cpp_cb";

const char CB_ON_RECEIVED_CB_FRONT[] =
R"__cpp_cb(
int $$::OnReceivedCB(const char* sender, const char* instance, rpc_port_h port, void *data)
{
  auto* cxt = static_cast<$$*>(data);
  rpc_port_parcel_h p;
  rpc_port_parcel_h result;
  int cmd;
  int ret;
  std::shared_ptr<ServiceBase> b;
  rpc_port_h callback_port;

  for (auto& i : cxt->services_) {
    if (i->GetInstance() == instance) {
      b = i;
      break;
    }
  }

  if (b.get() == nullptr) {
    _E("Failed to find $$ context(%s)", instance);
    return -1;
  }

  ret = rpc_port_stub_get_port(cxt->stub_, RPC_PORT_PORT_CALLBACK, instance,
      &callback_port);
  if (ret != 0) {
    _E("Failed to get callback port");
  }

  ret = rpc_port_parcel_create_from_port(&p, port);
  if (ret != 0) {
    _E("Failed to create parcel from port");
    return ret;
  }

  rpc_port_parcel_create(&result);
  rpc_port_parcel_read_int32(p, &cmd);

  switch (cmd) {
)__cpp_cb";

const char CB_ON_RECEIVED_CB_BACK[] =
R"__cpp_cb(    default:
      _E("Unknown command(%d)", cmd);
      rpc_port_parcel_destroy(p);
      rpc_port_parcel_destroy(result);
      return -1;
  }

  rpc_port_parcel_destroy(p);
  rpc_port_parcel_destroy(result);

  return ret;
}
)__cpp_cb";

const char CB_CTOR_SERVICE_BASE[] =
R"__cpp_cb($$::ServiceBase::ServiceBase(std::string sender, std::string instance)
    : sender_(std::move(sender)), instance_(std::move(instance)) {})__cpp_cb";

#endif  // IDLC_CPP_GEN_CPP_STUB_BODY_GEN_CB_H_
