/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <utility>

#include "idlc/element.h"
#include "idlc/type.h"

namespace tidl {

Element::Element(std::string id, BaseType* type,
                 std::string comments, unsigned line)
    : id_(std::move(id)), type_(type), comments_(std::move(comments)),
      line_(line) {}

const std::string& Element::GetID() const {
  return id_;
}

const BaseType& Element::GetType() const {
  return *type_;
}

const unsigned Element::GetLine() const {
  return line_;
}

const std::string& Element::GetComments() const {
  return comments_;
}

void Elements::Add(Element* elm) {
  elms_.emplace_back(elm);
}

const std::list<std::unique_ptr<Element>>& Elements::GetElms() const {
  return elms_;
}

bool Elements::Exist(Element* elm) const {
  for (auto& e : elms_) {
    if (e->GetID() == elm->GetID())
      return true;
  }

  return false;
}

}  // namespace tidl
