/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef IDLC_PARSER_H_
#define IDLC_PARSER_H_

#include <memory>
#include <string>
#include <list>

#include "idlc/document.h"

namespace tidl {

class Parser {
 public:
  Parser();
  ~Parser();

  void* Scanner() const { return scanner_; }
  bool Parse(const std::string& input);
  bool ParseFromFile(const std::string& path);
  void SetDoc(Document* doc);
  std::shared_ptr<Document> GetDoc();
  void ReportError(const std::string& err, unsigned line);

 private:
  void* scanner_;
  std::shared_ptr<Document> doc_;
  std::string path_;
  bool error_;
};

}  // namespace tidl

#endif  // IDLC_PARSER_H_
