/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <gtest/gtest.h>

#include <iostream>

#include "idlc/parser.h"
#include "idlc/cs_gen/cs_proxy_gen.h"

class CsProxyGenTest : public testing::Test {
 public:
  tidl::Parser* ps = nullptr;

  virtual void SetUp() {
    ps = new tidl::Parser();
  }

  virtual void TearDown() {
    delete ps;
  }

  bool FindStringFromFile(std::string filename, std::string str) {
    std::ifstream cb_file(filename);
    std::string line;

    while (std::getline(cb_file, line)) {
      if (line.find(str) == 0) {
        return true;
      }
    }

    return false;
  }
};

TEST_F(CsProxyGenTest, CsProxyGen_Constructor) {
  bool ret = ps->ParseFromFile("test.tidl");
  ASSERT_TRUE(ret);
  tidl::CsProxyGen gen(ps->GetDoc());
}

TEST_F(CsProxyGenTest, CsProxyGen_Run) {
  bool ret = ps->ParseFromFile("test.tidl");
  ASSERT_TRUE(ret);
  tidl::CsProxyGen gen(ps->GetDoc());
  gen.Run("test.cs");

  ASSERT_TRUE(FindStringFromFile("test.cs", "namespace RPCPort"));
  ASSERT_TRUE(FindStringFromFile("test.cs",
      "    public sealed class Student"));
  ASSERT_TRUE(FindStringFromFile("test.cs",
      "        public string name { get; set; }"));
  ASSERT_TRUE(FindStringFromFile("test.cs", "    public sealed class Class"));
  ASSERT_TRUE(FindStringFromFile("test.cs",
      "        public LinkedList<Student> students { get; set; }"));
}
